uniform float time;         // bazik preset playback time in seconds
uniform vec2 resolution;    // bazik render width (x) & height (y)

uniform vec3 color1;        // bazik user color 1 (r=x, g=y, b=z)
uniform vec3 color2;        // bazik user color 2 (r=x, g=y, b=z)
uniform vec3 color3;        // bazik user color 3 (r=x, g=y, b=z)
uniform vec3 color4;        // bazik user color 4 (r=x, g=y, b=z)

uniform float eq1;          // bazik 3 band equalizer bass value 0.0 to 1.0
uniform float eq2;          // bazik 3 band equalizer medium value 0.0 to 1.0
uniform float eq3;          // bazik 3 band equalizer high value 0.0 to 1.0
uniform float banger;       // bazik music banger (beat detect) 0.0 to 1.0

uniform int breaker;        // bazik user control 0 or 1
uniform int mode;           // bazik user control 1,2,3,4,5,6,7 or 8

uniform float par1;         // bazik user control -1.0 to 1.0
uniform float par2;         // bazik user control -1.0 to 1.0

// ---------------------------------------------
// Inspired by Mr Doom http://glsl.heroku.com/
// ---------------------------------------------


float Sphere_radius;

vec3 getSurfacePosition(vec2 coord) {
    
    float mov_radius;
    vec2  screen_center;
    vec3  Sphere_center;

    float index;
    vec2  tmpv2;
    float acc;
    
    index = 1.0;
    
    mov_radius = min(resolution.x, resolution.y) * (0.25 + (par1 * 0.3));
    screen_center = resolution * 0.5;
    acc = 0.0;
    
    for (int k=0; k< (2 + mode * 2); k++) {
        index += 0.2;
        Sphere_center = vec3(screen_center.x + mov_radius * (float(mode)*0.6)* sin(time*index),
           screen_center.y + mov_radius * cos(time*(5.0-index)*1.013),
           0.0);
        
        tmpv2 = Sphere_center.xy - coord.xy;
        acc += Sphere_radius/length(tmpv2);
    }
    if (acc > 1.00) {
        acc = 1.0 - 1.0/acc;
        acc *= Sphere_radius;
        
        acc = (Sphere_radius * acc)/(Sphere_radius*0.30 + acc);
        acc = (Sphere_radius * acc)/(Sphere_radius*0.30 + acc);
        acc = (Sphere_radius * acc)/(Sphere_radius*0.20 + acc);
        
        } else {
          acc = -200.0;
      }
      
      return vec3(coord, acc);
  }

  void main( void ) {

    vec2 screen_center;

    vec3 ViewDirection;
    ViewDirection = vec3 (.0,.0,-1.0);
    
    float mov_radius;
    
    mov_radius = min(resolution.x, resolution.y) * 0.4;
    screen_center = resolution * 0.5;
    
      if (breaker == 0){
          Sphere_radius = mov_radius * 0.1 - (float(mode) * 2.0) + eq1 * 10.0;
      }
      else {
          Sphere_radius = mov_radius * 0.1 - (float(mode) * 2.0);
      }
    
    /* Light Vars */

    vec3 LightColor;
    vec3 SpecularLightColor[3];
    vec3 DiffuseLightColor[3];
    vec3 AmbientLightColor;
    
    float DiffuseLightIntensity;
    float SpecularLightIntensity;
    float AmbientLightIntensity;
    
    DiffuseLightColor[0] = color1;
    SpecularLightColor[0] = vec3(1,1,1);
    
    DiffuseLightColor[1] = color2;
    SpecularLightColor[1] = vec3(1,1,1);
    
    DiffuseLightColor[2] = color3;
    SpecularLightColor[2] = vec3(1,1,1);
    
    AmbientLightColor = color4;
    
    vec3 LightPosition[3];
    vec3 LightDirection;
    vec3 SpecularDirection;
    
    /***********************************************************/
    /* Geometry                                                */
    /***********************************************************/
    vec3  SurfaceNormal;
    vec3  SurfacePosition;
    vec3  Sphere_center;
    vec3  SurfDX;
    vec3  SurfDY;
    vec2  tmpv2;
    
    float dist_from_center;
    
    dist_from_center = 2.0;
    
    float index;
    float inv;
    float acc;
    
    inv = .2;
    acc = .0;
    index = 1.0;
    
    LightColor = color1;
    
    SurfacePosition = getSurfacePosition(gl_FragCoord.xy);
    
    
    if (SurfacePosition.z > 0.0) { 
        SurfDX = getSurfacePosition(vec2(gl_FragCoord.x + 1.0, gl_FragCoord.y));
        SurfDY = getSurfacePosition(vec2(gl_FragCoord.x, gl_FragCoord.y+1.0));
        
        SurfDX.z *= 1.0;
        SurfDY.z *= 1.0;
        
        SurfaceNormal = normalize(cross(SurfDX - SurfacePosition, SurfDY - SurfacePosition));
        } else {
            SurfaceNormal = vec3(0.0,0.0,0.25);
        }

        
        /***********************************************************/
        
        LightColor = vec3(.0,.0,.0);
        
        LightPosition[0] = vec3(par2 * resolution.xy, Sphere_radius*3.0);
        LightPosition[1] = vec3(screen_center.x + cos(time*0.92) * mov_radius,
            screen_center.y - cos(time*1.011)* mov_radius,
            Sphere_radius* 8.0 * cos(time*1.013));
        LightPosition[2] = vec3(screen_center.x + sin(time*1.019) * mov_radius,
            screen_center.y - sin(time*1.209) * mov_radius,
            Sphere_radius* 4.0 * (1.0 + cos(time*1.007)));
        
        for (int i=0; i<3; i++) {
            LightDirection = normalize(LightPosition[i] - SurfacePosition);
            
            /***********************************************************/
            /* Diffuse Light                                           */
            /***********************************************************/
            
            DiffuseLightIntensity = clamp(dot(LightDirection, SurfaceNormal),.0,1.0);
            
            /***********************************************************/
            /* Specular Light                                          */
            /***********************************************************/
            
            SpecularDirection = LightDirection - 2.0 * SurfaceNormal * dot(LightDirection, SurfaceNormal);
            
            SpecularLightIntensity = clamp(dot(ViewDirection, SpecularDirection),.0,1.0);
            
            SpecularLightIntensity *= SpecularLightIntensity*SpecularLightIntensity;
            SpecularLightIntensity *= SpecularLightIntensity*SpecularLightIntensity;
            SpecularLightIntensity *= SpecularLightIntensity*SpecularLightIntensity;
            SpecularLightIntensity *= SpecularLightIntensity*SpecularLightIntensity;

            LightColor += DiffuseLightColor[i] * DiffuseLightIntensity +
            SpecularLightColor[i] * SpecularLightIntensity;
            
            AmbientLightIntensity += clamp(DiffuseLightIntensity,.0,.5);
        }
        
        
        /***********************************************************/
        /* Ambient Light                                           */
        /***********************************************************/

        AmbientLightIntensity = 1.0 - AmbientLightIntensity;
        
        
        /***********************************************************/
        /* Lighting Finalizing                                     */
        /***********************************************************/
        
        LightColor += AmbientLightIntensity*AmbientLightColor;
        
        gl_FragColor = vec4(LightColor,1.0);
      
      //gl_FragColor.r = color1.x;
      //gl_FragColor.g = color1.y;
      //gl_FragColor.b = color1.z;
      //gl_FragColor.a = 1.0;
    }